/**
 * MCS Media Computer Software
 * Copyright 2019 by Wilfried Klaas
 * Project: SPSEmulator
 * File: TestRESTEndpoints.java
 * EMail: W.Klaas@gmx.de
 * Created: 03.02.2019 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.emulator;

import static org.junit.Assert.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.IOException;
import java.io.InputStream;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Set;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;

import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.emulator.model.CommandModel.COMMAND;
import de.mcs.tools.sps.emulator.model.WebSessionModel;

/**
 * @author wklaa_000
 *
 */
class TestRESTEndpoints {

  private static Invocation.Builder invocationBuilder;

  @BeforeAll
  public static void init() throws KeyManagementException, NoSuchAlgorithmException {
    TrustManager[] noopTrustManager = new TrustManager[] { new X509TrustManager() {

      @Override
      public X509Certificate[] getAcceptedIssuers() {
        return null;
      }

      @Override
      public void checkClientTrusted(java.security.cert.X509Certificate[] certs, String authType) {
      }

      @Override
      public void checkServerTrusted(java.security.cert.X509Certificate[] certs, String authType) {
      }
    } };

    javax.net.ssl.HttpsURLConnection.setDefaultHostnameVerifier(new javax.net.ssl.HostnameVerifier() {

      public boolean verify(String hostname, javax.net.ssl.SSLSession sslSession) {
        return true;
      }
    });

    SSLContext sc = SSLContext.getInstance("ssl");
    sc.init(null, noopTrustManager, null);

    Client client = ClientBuilder.newBuilder().sslContext(sc).build();
    WebTarget webTarget = client.target("https://127.0.0.1:8443/emulator");
    invocationBuilder = webTarget.request(MediaType.APPLICATION_JSON);
  }

  /**
   * @throws java.lang.Exception
   */
  @BeforeEach
  void setUp() throws Exception {
  }

  @Test
  void test() throws IOException {
    WebSessionModel webSessionModel = invocationBuilder.get(WebSessionModel.class);
    assertNotNull(webSessionModel);
    System.out.println(webSessionModel.toString());

    try (InputStream inputStream = ClassLoader.getSystemResourceAsStream("test.tps")) {
      List<String> readLines = IOUtils.readLines(inputStream, "UTF-8");
      assertNotNull(readLines);
      webSessionModel.getProgram().setSource(readLines.toArray(new String[0]));
    }

    Set<COMMAND> availableCommands = webSessionModel.getCommand().getAvailableCommands();
    assertNotNull(availableCommands);
    assertTrue(availableCommands.contains(COMMAND.COMPILE));
    webSessionModel.getCommand().setActualCommand(COMMAND.COMPILE);
    webSessionModel = invocationBuilder.post(Entity.entity(webSessionModel, MediaType.APPLICATION_JSON),
        WebSessionModel.class);
    assertNotNull(webSessionModel);
    assertTrue(webSessionModel.getProgram().getBin().length > 10);
    System.out.println(webSessionModel.toString());
  }

}
